import sys
import requests
import logging

# Nastavení loggeru
logging.basicConfig(
    filename='/var/log/portfolio.log',
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s'
)

logging.info('Skript spuštěn.')

MOODLE_URL = "https://portfolio.zcu.cz/webservice/rest/server.php"
API_TOKEN = ""
STUDENT_TEMPLATE_ID = 4  # ID kurzu pro studenty
TEACHER_TEMPLATE_ID = 5  # ID kurzu pro ostatní
EDITING_TEACHER_ROLEID = 3  # ID role 'editingteacher'

def duplicate_course(source_courseid, categoryid, fullname, shortname):
    """Vytvoření kopie kurzu přes API a získání ID nového kurzu."""
    params = {
        "wstoken": API_TOKEN,
        "wsfunction": "core_course_duplicate_course",
        "moodlewsrestformat": "json",
        "courseid": source_courseid,
        "fullname": fullname,
        "shortname": shortname,
        "categoryid": categoryid,
        "visible": 1  # Kurz bude rovnou viditelný
    }
    response = requests.post(MOODLE_URL, data=params)
    response_json = response.json()

    if "exception" in response_json:
        logging.error(f"Chyba při kopírování kurzu {source_courseid}: {response_json}")
        raise Exception(f"Chyba při kopírování kurzu {source_courseid}: {response_json}")

    new_courseid = response_json.get("id")
    if not new_courseid:
        logging.error("API nevrátilo ID nového kurzu.")
        raise Exception("API nevrátilo ID nového kurzu.")

    logging.info(f"Nový kurz vytvořen s ID: {new_courseid}")
    return new_courseid

def get_course_contextid(courseid):
    """Získání contextid pro kurz přes API."""
    params = {
        "wstoken": API_TOKEN,
        "wsfunction": "core_course_get_courses_by_field",
        "moodlewsrestformat": "json",
        "field": "id",
        "value": courseid
    }
    response = requests.get(MOODLE_URL, params=params)
    response_json = response.json()
    
    if "exception" in response_json:
        logging.error(f"Chyba při získávání contextid pro kurz {courseid}: {response_json}")
        raise Exception(f"Chyba při získávání contextid pro kurz {courseid}: {response_json}")
    
    courses = response_json.get("courses", [])
    if courses:
        contextid = courses[0]["id"]
        logging.info(f"Získán contextid {contextid} pro kurz ID {courseid}.")
        return contextid
    else:
        logging.error(f"Kurz s ID {courseid} nebyl nalezen.")
        raise Exception(f"Kurz s ID {courseid} nebyl nalezen.")

def assign_role(userid, courseid, roleid):
    """Přiřazení role uživateli v kurzu."""
    params = {
        "wstoken": API_TOKEN,
        "wsfunction": "core_role_assign_roles",
        "moodlewsrestformat": "json",
        "assignments[0][roleid]": roleid,
        "assignments[0][userid]": userid,
        "assignments[0][contextid]": get_course_contextid(courseid)
    }
    response = requests.post(MOODLE_URL, data=params)
    if response.status_code == 200 and "exception" not in response.text:
        logging.info(f"Role ID {roleid} přiřazena uživateli ID {userid} v kurzu ID {courseid}.")
    else:
        logging.error(f"Chyba při přiřazování role: {response.text}")
        raise Exception(f"Chyba při přiřazování role: {response.text}")

def enrol_user_to_course(userid, courseid):
    """Zápis uživatele do kurzu přes API."""
    params = {
        "wstoken": API_TOKEN,
        "wsfunction": "enrol_manual_enrol_users",
        "moodlewsrestformat": "json",
        "enrolments[0][roleid]": EDITING_TEACHER_ROLEID,
        "enrolments[0][userid]": userid,
        "enrolments[0][courseid]": courseid
    }
    response = requests.post(MOODLE_URL, data=params)
    if response.status_code == 200 and "exception" not in response.text:
        logging.info(f"Uživatel {userid} úspěšně zapsán do kurzu {courseid}.")
    else:
        logging.error(f"Chyba při zápisu uživatele {userid} do kurzu {courseid}: {response.text}")
        raise Exception(f"Chyba při zápisu uživatele {userid} do kurzu {courseid}: {response.text}")

def main():
    if len(sys.argv) != 5:
        logging.error("Nesprávný počet argumentů. Očekává se: userid, fullname, shortname, categoryid.")
        sys.exit(1)

    userid, fullname, shortname, categoryid = sys.argv[1:5]
    categoryid = int(categoryid)

    try:
        source_courseid = STUDENT_TEMPLATE_ID if categoryid == 9 else TEACHER_TEMPLATE_ID
        courseid = duplicate_course(source_courseid, categoryid, fullname, shortname)
        assign_role(userid, courseid, EDITING_TEACHER_ROLEID)
        enrol_user_to_course(userid, courseid)
    except Exception as e:
        logging.error(f"Chyba během provádění skriptu: {e}")
        sys.exit(1)

if __name__ == "__main__":
    main()
